# General lab info

In this course you will use Git extensively. To master Git and/or similar content management system with version control is absolutely essential engineering skill. If you have not used Git or similar, here is your chance to learn.

If you are under Linux, you likely have Git installed by default (else use your Linux package manager to install it). If you are under Windows, then look at https://git-scm.com/downloads. If you get stuck, use the Telegram group to ask questions and help each other out.

In general:

- Discussions encouraged (use Telegram/Zoom etc.)
- Labs are done individually, it should be your "original" work
- Create a user on https://gitlab.henriktjader.com/ (if you don't have an account there already)

For each lab:

- Fork the lab repository (if not done already)
- Clone your repository to your local machine (if not done already)
- Solve the lab assignment(s)
- Commit your repository
- Submit link to your repository as a Canvas assignment (you will then be assigned two repositories to review)
- Make reviews as GitLab Issue(s)
- Submit links to the repositories you have reviewed as a Canvas assignment.
- Attend the reviews/issues you have received
- Submit link to you final repo as a Canvas assignment.

Pay strict attention to the deadlines (the course depends on everyone being on time, to give and get reviews etc.). If you get sick and due to that cannot complete an assignment in time, give us a heads up in Telegram, and we will try to find a solution.

Hint: Remember to commit often, this allows you to rollback to previous states. Name your commits so that it reflects the recent changes, then your git log will make much more sense, and will be easier to navigate.

Hint 2: This repository will be updated throughout the course, so make sure to "pull" the latest version. Check the CHANGELOG.md for update info.

---

## Lab 1 (Schematics)

The main topic of this lab is to give you a good understanding on how to use the information in _datasheets_ to draw components and connect them in the schematic view.

## Task 1

Read the whole lab instruction before you start.

Design and connect all necessary components to the microcontroller (MCU). Some of the components can’t be found in the standard libraries so you have to draw some of them by yourself and extract the necessary information through datasheets. The work flow of KiCad:

- Add components and draw the schematic of your design
- Assign a footprint to your schematic components
- Generate a net-file
- Layout (Lab2)

### 1) Adding the microcontroller

Start ”KiCad” and follow the instructions:

- Make a new project.
- Open the schematic. (double-click on the \*.sch-file)
- Click on Preferences/Manage symbols libraries.
- Check the library `MCU_ST_STM32F4` is active in your project.
- Click on the ”Place symbol”-button to the right and then click on the sheet where you want the component to be placed.
- Find your microcontroller (`STM32F411RETx`) in the stm32-library.
- You have now added the microcontroller to your project. It needs additional components to make it run, e.g. a crystal (for stable USB clocking), decoupling capacitors, power and a SWD (Serial Wire Debug) connector to program it.

### 2) Decoupling Capacitors

To stabilize the voltage at the power pins on the MCU, decoupling capacitors must be provided. Use at least one small package SMD MLCC capacitor on each of the power supply pins, with a capacitance of at least 100nF.

This microcontroller also needs additional decoupling capacitors for its internal voltage regulator and analog peripherals, you can find the datasheet by searching on internet (Hint: ”STM32F411 datasheet”).

- Open the datasheet and look at the figure in section 6.1.6.
- Add capacitors to the power pins as the figure shows.  
  (Place component-button, click on sheet, device/C)
- Hoover over the capacitor, press ”V” and give the capacitor its value e.g. 100n.
- Connect them to the microcontroller using the ”wire” button. (W key)

You should have at least 8 capacitors connected to the microcontroller.

### 3) Adding supply symbols

VDD should be connected to +3.3V and VSS should be connected to ground. These nets are often used in a schematic, so to avoid drawing nets across the whole schematic you can add power ports. All power ports are global in your design, which means that all power ports of the same kind is connected to each other.

- Click on the Place power port button.
- Select ”+3.3V”.
- Add power ports on appropriate places and connect them to the pins and capacitors that should have 3.3 volt.
- Do the same thing with GND.
- Note that the VCAP1 pin should NOT be connected to +3.3V.

### 4) NRST Reset pin

The NRST pin is an active low (0) input used to reset the MCU.
There is an internal pull-up resistor keeping the pin high (allowing the MCU to run). For more info on what a pull-up resistor is, search the internet for ”Pull-up resistor”.

Later we will add a Reset Button, so that you can force reset even when the MCU is powered and running.

### 5) Pull-down resistor to BOOT0 (boot mode)

The microcontroller can boot in different modes, similar to how you can enter the BIOS when booting your PC by pressing the F2-key (or other F-key). This can save you in tricky situations, for example if you are playing with the main clock or with sleep modes and it crashes, making you unable to upload (flash) a new binary to the microcontroller. You can boot in "safe mode" by pulling this pin high (so it does not run you code on startup) allowing you to upload a new binary even if your code would otherwise brick your MCU.

- Add a 10k pull-down resistor to the BOOT0 pin.
- Add a "jumper" between BOOT0 and +3.3v, so that you can easily boot into "safe mode" by shorting these two terminals. You could make this a 3-pin header and "no-connect" the third terminal in order to provide a place to store the jumper when not in use.

### 6) Crystal

The crystal will resonate at a specified frequency. Its waveform will be used as an input to the PLL (Phase Locked Loop) that will multiply the crystal’s frequency. This is the crystal that will be used: https://www.elfa.se/sv/kristall-ytmontering-16mhz-iqd-lfxtal029370/p/30131284

The crystal and its associated capacitors are part of a "Pierce oscillator" circuit connected to two special pins on the MCU - `PH0 - OSC_IN` and `PH1 - OSC_OUT`. To calculate value of the load capacitors required for proper operation, use the following formula:

`C = 2*(Cload - Cstray)`

where `Cload` is the load capacitance value specified in the crystal's [datasheet](https://www.elfa.se/Web/Downloads/_t/ds/LFXTAL029370REEL_eng_tds.pdf) (18pF) and `Cstray` is the stray capaciatance on the PCB, usually in the range of 2-10pF (pick something in-between, like 6pF). This gives us a capacitance value of `2*(18 - 6) = 24pF`. The closest standard value would be `22pF`.

- Click on the Place component button.
- Find the device/Crystal.
- Place it close to `PH0 - OSC_IN` and `PH1 - OSC_OUT`, pin 5 and 6.
- Connect the component with wires to the mentioned pins. Value 16MHz.
- Connect one 22pF capacitor to each pin on the crystal and connect the other end of the capacitors to GND.

### 7) Serial Wire Debug (SWD) connector

To program and debug the MCU we will use a SWD connector (6 pin header). Sometimes it can be tricky to find all the information in the datasheet and user manual for the microcontroller, all the information you need is there, it will just take time to find it. However, it can then be useful to follow proven examples like your Nucleo board. This is the type of connector that will be used: https://www.elfa.se/en/straight-male-pcb-header-through-hole-rows-contacts-54mm-pitch-rnd-connect-rnd-205-00627/p/30093647

- Add one conn/CONN01X06 (male).
- Search for the Nucleo boards datasheet.
- In this datasheet search for the table that explains the connector `CN4` pinout. You can also investigate how they have connected the target MCU to these pins in the schematic below in the datasheet.
- You can connect pins without wires going everywhere. Use ”net labels” (button to the right) to name one wire. It will connect all wires with the same name in the local sheet.
- VDDTARGET→+3.3V. GND→GND. NRST→NRST. SWCLK→SWCLK (PA14). SWDIO→SWDIO (PA13). SWO→SWO (PB3)

### 8) Power from USB

One easy way to get power to your project is through USB. It can deliver up to 100mA without any negotiations.

- Add a ”USBOTG” connector to your schematic. Already here you can think about how you should connect your mouse and use a suitable connector (mini, micro or even usb-c).
- Find the USB pinout for your connector.

### 9) 3.3V Power Supply

The MCU wants to be powered from a 3.3V rail. To this end we will use a fixed LDO (low dropout) linear regulator, to convert a 5V input (from the USB connector) to a 3.3V output. We will use the [AP2125N-3.3TRG1](https://www.elfa.se/en/ldo-voltage-regulator-3v-360ma-sot-23-diodes-incorporated-ap2125n-3trg1/p/30041163) for this.

Look in the [datasheet](https://www.elfa.se/Web/Downloads/_t/ds/AP2125_eng_tds.pdf) for a typical application circuit. Later we will design the power supply for the PWM3389 sensor, which is a bit trickier.

### 10) PWM3389DM-T3QU Schematic Symbol

Download the general [datasheet](https://www.pixart.com/products-detail/4/PMW3389DM-T3QU), and look at section 3. There you will find a reference schematic.

For some components (like the PMW3389DM-T3QU) there might not exist a pre-made library, so you will now learn how to draw your own component.

- Click on Tools/ Symbol Library Editor.
- Create a new library for your project and click on Create New symbol in the new library).
- Component name ”PMW3389”. OK.
- Add pins and name them according to the datasheet. Remember to assign the correct pin number as well.
- Make sure to set the ”Electrical type” on each pin according to usage (signal in/out, power in/out etc.) `X` marks a non connected pin. In our setting we don't use `GPO` so you can set it `X`.
- Save by clicking on the ”Save current component to new library”-button (found at the top). This is only done for the first component, you can add several components to the same lib later.
- Add the newly created component to your schematic.

Hint. Never, never ever, change the grid settings when placing the pins in the symbol.

### 11) 1.9V Power Supply

As you can see in the PWM3389DM-T3QU datasheet and reference schematics, the sensor requires a 1.9v power supply. This is a much less common voltage for a fixed output LDO (try finding one from Elfa or rs-online and you will likely fail).

To that end, you can use an adjustable voltage regulator, such as [MIC5235YM5-TR](https://www.elfa.se/en/ldo-voltage-regulator-24-20v-sot-23-microchip-mic5235ym5-tr/p/30099685), which will let you set your desired output voltage using external resistors. The circuit and equation for calculating the resistor values needed to acheive the desired the output voltage (1.9V) can be found in the [datasheet](https://www.elfa.se/Web/Downloads/_t/ds/MIC5235YM5-TR_eng_tds.pdf) for the regulator. Here you will also find information about input- and output capacitor values needed for proper operation.

Once you have added the 1.9v power supply you can add a new power net, and connect that to your PMW3389 component.

### 12) Reset button

Now add a reset button. Suitable value for capacitor is found in the MCU data sheet.

### 13) Power LEDs and test points

In order to see that your system is correctly powered and to have easy access to probing your signal lines etc, it is recommended to add test points to your board in form of exposed pads. Preferably use a pin header footprint for your GND test points however, so you can clip the scope probe's GND alligator clip to it. Provide at least one GND test point for each subcircuit block on your boards. You don't need to stuff them with pins by default during assembly, but it is nice to have the option to add those in when troubleshooting.

It is also recommended you add LEDs to your power rails so you can see they're powered up correctly. You can use an LED such as [this red one](https://www.elfa.se/en/smd-led-645nm-0805-65cd-30ma-kingbright-kptd-2012surck/p/30118951), together with a suitable current limiting resistor. The forward voltage drop of a red LED is around 1.8V, aim for a forward current of about 2mA. You can use this [LED calculator](https://ohmslawcalculator.com/led-resistor-calculator) to calculate suitable resistor values for your LEDs. Use resistor values from the E12 series.

### 14) SPI

SPI (Serial Peripheral Interface) is a common way to interconnect components synchronized with a clock signal. In our case the STM32F401/F411 will be the Master and the PMW3389 will be the slave (so all communication is driven by the MCU).

In order to allocate pins to peripherals there is a nice tool [STM32CubeMX](https://www.st.com/en/development-tools/stm32cubemx.html). Install STM32CubeMX (either using the link, or your package manager), select the target (smt32f401, e.g.) and find suitable (unused pins for) SPI communication. E.g., using SPI2 you can assign.

```shell
// Configure SPI
// spi2
// sck    - pb10,
// miso   - pc2,
// mosi   - pc3,
// ncs    - pb12,
```

Decide on which pins you want to use, and connect the PMW3389 to your MCU. You can connect the `Motion` signal to a free GPIO (its used to generate an interrupt when the sensor detects a motion event, for allowing wake from sleep mode). Likely you don't need this but you can add it in any case.

### 15) USB Data

Until now you have used the USB only to power the system, but you will need to add data lines +D/-D as well for communication. Check the data sheet for specifics.

You may also want to protect your system from ESD (Electrostatic discharge). To that end you may optionally add [ESD protection circuitry](https://www.elfa.se/en/esd-protection-diode-bi-directional-25v-sot-23-st-usblc6-2sc6/p/30034148).

### 16) Assign Footprint

It is very important that the pin numbering on your component is correct to the footprint in the library! Always double check with the datasheet. You will now associate each component to a footprint.

- Annotate all the components, this will give your components a name.
- Click on the ”Assign PCB footprints to schematic symbols” to associate components and footprints”-button at the top.
- Click on the View selected footprint button at the top, it will display the footprint in question.
- We recommend you to use 0805-size capacitors and resistors. Associate all the res. and ceramic caps. to 0805 respectively, Handsoldering. (Capacitors SMD and Resistors SMD)
- Read in the datasheet or on the link for the voltage regulator and select the correct footprint. Make sure that all components are in stock for immediate delivery.
- Repeat for the crystal and SWD connector.
- MCU = HousingsQFP:LQFP-6410x10mmPitch0.5mm.
- USBOTG = Connect:USBMini-B (or to your liking)
- For the PMW3389 we will make a custom footprint later, so leave this one unassigned for now.
- Don’t forget to save!

  Hint: Check that the numbering of the pins are correct! Hint 2: When you are working with your project later, you will probably not find all footprints in the standard library. There are good tutorials on youtube on how to draw your ownfootprints.

### 17) Electric Rule Check (ERC)

ERC is a check that detects if you have any ”loose ends”, meaning that it will give you a warning if you have a net with only one connection. It will also give you a warning if you have drawn a net to a component but the net and the component haven’t connected. To get the ERC to work as intended, you have to mark all the unused pins with a Not-connected flag, found to the right.

- 1. ”Perform electrical rules check” button (Ladybug button).
- 2. Do an ERC.
- 3. Fix errors.
- 4. Repeat point 1 until errors == 0 && warnings == 3. The GND net is not connected to a Power output, the 5 voltage neither and the capacitor connected to VCAP1 is not connected to a power output.

## First Submission

In Canvas look for Lab1-First Submission, and submit a link to your repo there before the deadline.

## Review

You will be assigned two repositories to review. Clone the repositories and compare to your solution.

Check each that each of the 17 steps has been correctly conducted and that the calculated values for generating 1.9v makes sense, and that the LED selection/current limitation is within specs.

Make an issue in each repository that you review with detailed comments to the author/repo owner. Later follow up on those comments and close them when your issues have been addressed to your satisfaction.

In Canvas look for Lab1-Review, and submit links to the repositories you have reviewed before the corresponding deadline.

## Final

Address the raised issues, and commit your fixes. This is an iterative process, where you help each other coming up with a working solution by leading questions, hints etc. (Just giving the answer directly is not particularly fruitful, its a learning process right.)

Once you have addressed all issues, look in Canvas for Lab1-Final, and submit a link to your repository.

---

## Lab 2 (Layout)

The main topic of this lab is to give you a good understanding on how to place the components on the Printed Circuit Board (PCB) and route the traces.

## Task 2

Read the whole lab instruction before you start.

## 1) Creating a foot-print for the PMW3389DM

Most components come with a ready made KiCad footprint (in either the stock libraries or some 3rd party library, e.g. [snapeda](https://www.snapeda.com)). However, for the PMW3389DM you need to create your own. Check in the [data-sheet](https://www.google.com/url?sa=t&rct=j&q=&esrc=s&source=web&cd=&ved=2ahUKEwiHvuC1trXuAhXFk4sKHfqDA-cQFjAAegQIARAC&url=https%3A%2F%2Fwww.pixart.com%2F_getfs.php%3Ftb%3Dproduct%26id%3D4%26fs%3Dck2_fs_cn&usg=AOvVaw1A1rR533Pt-7EgnVSS-_ch), section 2,3 for detailed geometry. Use the footprint editor (`Edit PCB Footprints`) to make a [footprint](https://forum.kicad.info/t/tutorial-how-to-make-a-footprint-in-kicad-5-1-x-from-scratch/11092).

Set the grid to fit the spacing between the pins (1.78mm/0.070 inch) between pins, so a grid at 0.035 inch works. Notice, the sensor uses a skewed non standard offset between the rows (offset by 0.89mm/0.035 inch). Spacing between rows are 14.48mm/0.570 inch (unfortunately this won't match the grid though).

Once you have made the footprint save it and go back to the schematic and associate the PMW3389 to your newly made footprint.

## 2) Design rules

You will now create a layout for your project. This is where you physically place the components on the PCB and draw the copper traces to connect the components. Before you start placing the components you have to setup some "design rules" for the layout so the manufacturer can make the PCB.

We use Aisler, and their 2-layer "Beautiful Boards" with [specs](https://aisler.net/help/design-rules-and-specifications/specifications). Aisler directly accepts KiCad designs, which is great as we don't need to worry about exporting to other formats (such as Gerber). Moreover, there are ready made templates for setting up your KiCad project to fit their manufacturing process,  [KiCad-Templates](https://github.com/sethhillbrand/kicad_templates).

KiCad uses text files both for storing configuration parameters and your actual design. (Normally its a bad idea to fiddle with machine generated files manually, but they are actually quite readable and fairly easy to understand/navigate.)

- Start KiCad and open your project.
- Open the schematic of the project and generate a netlist-file, see buttons at the top.
- Click on the button ”Run Pcbnew to layout printed circuit board”, (see buttons at the top).
- Make sure that the grid is set to ”mm” and change the grid size to 1,0mm.
- Click on the Read netlist button and select your newly generated netlist-file, click Read Current Netlist.

- Click on the menu File/Board Setup/Design Rules, you should set:
  
| Requirement          | Value   |
| -------------------- | ------- |
| Minimum track width  | 0,2mm   |
| Minimum via diameter | 0,4mm   |
| Minimum via drill    | 0,3mm   |
| Minimum hole to hole | 0,25 mm |

(We don't use micro vias in our designs.)  
  
- Under "Net Classes" we add a new class. Click on the ”Add” button to add a classes named Power, and set the track width to 0,5mm. We want this net to handle more current, hence we use a wider trace.

- Transfer GND, 1.9V, 3.3V and 5V to the (new) Power net class. Now you can close the Board Setup (and return to it later if you want to change some setting).

## 3) Place you components

- Select the layer `Edge.Cuts`, found to the right, and draw a rectangle 60mm x 40mm.
  
The 60mm x 40mm rectangle is your workspace for this lab. When you are working with your project you need to adopt it to fit into your mouse shell.

You will see that after reading in the netlist, your components are just lumped together so you will need to place your components in the yellow square that you crated earlier.

Rotate and move around the components and try to minimize
the crossed paths of the un-routed lines (gray lines). All components should fit inside the 60mm x 40mm rectangle.

Common practice for placing your components:

- Overall layout: Where you want your connectors, buttons etc. dictates the layout. For now, the USB and PWM3389 is important, later buttons/wheels, additional LEDs etc. need to be carefully considered. Check, e.g., how mice sensors are typically placed relative to the center/hand placement.

- Components are typically designed with layout in mind, consider the orientation/placement to reduce routing complexity. (E.g., inputs are grouped on one edge, outputs on another. Analog pins are grouped allowing to split ground planes etc.)
  
- Most datasheets will give an example layout for decoupling and interconnects, these are helpful to give an initial idea.

- Place your MCU and it’s decoupling capacitors first. The decoupling capacitors should always be near it’s pins, this is needed to eliminate power-noise to the MCU.

- Place other noise sensitive components, then finally the rest of the components.

Hint! You can uncheck unnecessary layers and just keep ”F/B.Cu”, ”F/B.SilkS” and Edge.Cuts. If you have the schematic window zoomed in next to the layout window, you can see which component you have selected and it will make it easier to know which is what.

## 4) Route the traces

You will now draw where the copper should be placed, red means top layer and green means bottom layer.

- Change the grid size to 0,1mm.
- Click on the button ”Route tracks”.
- Click on a pad you could like to connect, press ”V” if you need to do a via, route your way to the another pad.
- Do this for all gray lines. (Unconnected == 0)
  
Hint! While having the routing tool selected, right click/Routing options... and change between Shove and Walk around.

No more gray lines?

- Select front copper layer.
- Draw a filled zone in the rectangle, Associate it to the GND net.
- Do the same thing for the back layer.

You have now created two ground planes, one one each side of the PCB. Alternately you may have a power power plane and a ground plane. You may activate to `Show filled areas in zones`, to see where copper will be poured.

Placement and routing is an iterative process, you will likely need to try different options/approaches until you have your final layout.

## 5) Cut-out for the PMW3389DM

The sensor requires a cut-out, look in the data-sheet for exact dimensions. You can use the edge cut layer to make the cut out. Alternatively, you can make the edge cut a part of the footprint (but I have never done that, so not sure how hard it is, [see](https://forum.kicad.info/t/create-a-custom-shape-hole/12646)). Activate to `Show filled areas in zones`, to see verify that you don't pour copper under the the sensor.

## 6) Design Rule Check (DRC)

The Design Rule Check give you warnings if you don’t comply with the Design rules that you previously setup. It will give you warnings on short circuits, clearance issues etc..

- 1 Click on Tools/DRC, see that the options comply to the manufacturers specification.
- 2 Fix errors.
- 3 Repeat point 1 until errors == 0.

---

## First Submission Lab 2

In Canvas look for Lab2-First Submission, and submit a link to your repo there before the deadline.

## Review Lab 2

You will be assigned two repositories to review. Clone the repositories and compare to your solution.

Check each that each of the 6 steps has been correctly conducted, and comment on possible problems with the layout.

Make an issue in each repository that you review with detailed comments to the author/repo owner. Later follow up on those comments and close them when your issues have been addressed to your satisfaction.

In Canvas look for Lab2-Review, and submit links to the repositories you have reviewed before the corresponding deadline.

## Final Lab 2

Address the raised issues.

Once you have addressed all issues, look in Canvas for Lab2-Final, and submit a link to your repository.